from openbrain.orm.model_client import Client
from ob_tuner.util import logger
import gradio as gr

def save_client_details(email, leadmo_api_key, leadmo_location_id ,leadmo_calendar_id, first_name, last_name, date_of_birth, phone, address1, address2, city, state, country, postal_code, website):
    """Update the client with the latest values from the database"""
    try:
        client = Client(
            email=email,
            leadmo_api_key=leadmo_api_key,
            leadmo_location_id=leadmo_location_id,
            leadmo_calendar_id=leadmo_calendar_id,
            first_name=first_name,
            last_name=last_name,
            date_of_birth=date_of_birth,
            phone=phone,
            address1=address1,
            address2=address2,
            city=city,
            state=state,
            country=country,
            postal_code=postal_code,
            website=website
        )
        client.save()
        gr.Info(f"Successfully updated ")
    except Exception as e:
        gr.Error(f"Please enter your Lead Momentum API key and location ID in order to save.")

def load_client_details(email):
    """Load the client details from the database"""
    try:
        _client = Client.get(email=email)
    except Exception as e:
        gr.Warning(f"Please enter your Lead Momentum API key and location ID, then save your profile to begin using Lead Momentum tools.")
        return ["" for _ in range(14)]
        # _client = Client(email=email, leadmo_api_key="", leadmo_location_id=None, lls_api_key=None)
        # _client.save()

    leadmo_api_key = _client.model_dump().get('leadmo_api_key', "")
    leadmo_location_id = _client.model_dump().get('leadmo_location_id', "")

    leadmo_calendar_id = _client.model_dump().get('leadmo_calendar_id', "")
    first_name = _client.model_dump().get('first_name', "")
    last_name = _client.model_dump().get('last_name', "")
    date_of_birth = _client.model_dump().get('date_of_birth', "")
    phone = _client.model_dump().get('phone', "")
    address1 = _client.model_dump().get('address1', "")
    address2 = _client.model_dump().get('address2', "")
    city = _client.model_dump().get('city', "")
    state = _client.model_dump().get('state', "")
    country = _client.model_dump().get('country', "")
    postal_code = _client.model_dump().get('postal_code', "")
    website = _client.model_dump().get('website', "")

    return [leadmo_api_key, leadmo_location_id, leadmo_calendar_id, first_name, last_name, date_of_birth, phone, address1, address2, city, state, country, postal_code, website]



import json
import re
from decimal import Decimal
from json import JSONDecodeError

import faker
import gradio
from botocore.exceptions import NoCredentialsError

from ob_tuner.util import *
from openbrain.orm.model_client import Client
import openbrain.orm
import openbrain.util
from openbrain.agents.gpt_agent import GptAgent
from openbrain.orm.model_agent_config import AgentConfig
from openbrain.orm.model_chat_message import ChatMessage
from openbrain.orm.model_chat_session import ChatSession
from openbrain.util import config, Defaults

def get_aws_cloudwatch_logs(_session_state=None):
    """Get cloudwatch logs for the agent we are interacting with"""

    logger.info("Getting AWS Cloudwatch logs...")
    if not OB_PROVIDER_API_KEY:
        return "Local mode... no API to monitor."
    try:
        cloudwatch = boto3.client("logs")
        target_log_group_name_prefix = f"/aws/lambda/{INFRA_STACK_NAME}-APIHandler"
        log_groups = cloudwatch.describe_log_groups(logGroupNamePrefix=target_log_group_name_prefix)

        logger.info(f"Found {len(log_groups['logGroups'])} log groups.")
        target_log_group = log_groups["logGroups"][0]["logGroupName"]
        logger.info(f"Target log group: {target_log_group}")

        if not target_log_group:
            return "No log group found"

        # get the log streams for the log group
        log_streams = cloudwatch.describe_log_streams(logGroupName=target_log_group, orderBy="LastEventTime", descending=True)

        # get the latest log stream
        latest_log_stream = log_streams["logStreams"][0]["logStreamName"]

        # get the log events
        log_events = cloudwatch.get_log_events(logGroupName=target_log_group, logStreamName=latest_log_stream)

        events_string = ""
        # match_prefix = '{"level":"'

        # target_xray_trace_id = _session_state['last_response'].headers['x-amzn-trace-id']
        # target_xray_trace_id = re.sub(r';.*', '', target_xray_trace_id.replace("Root=", ""))
        max_events = 10
        counter = 0
        for event in log_events["events"]:
            message = event["message"]
            # if target_xray_trace_id in message:
            try:
                message_dict = json.loads(message, parse_float=Decimal)

                new_dict = {
                    "level": message_dict.get("level", None),
                    "location": message_dict.get("location", None),
                    "message": message_dict.get("message", None),
                    "timestamp": message_dict.get("timestamp", None),
                    "function_name": message_dict.get("function_name", None),
                    "request_path": message_dict.get("request_path", None),
                    "xray_trace_id": message_dict.get("xray_trace_id", None),
                }
            except JSONDecodeError:
                continue
            except KeyError:
                continue

            message = json.dumps(new_dict, indent=2)

            events_string += message + ",\n"
            counter += 1
            if counter > max_events:
                break

        # remove the last comma
        try:
            events_string = events_string[:-2]
        except Exception:
            events_string = json.dumps({"Idle": "No events found"})
        # formatted_message = '''```python\n''' + events_string + '''\n```'''
        formatted_message = "[\n" + events_string + "\n]"
        return formatted_message
    except Exception as e:
        return json.dumps({"exception": e.__str__()})


def chat(message, chat_history, _profile_name, _session_state, _client_id, _context):
    # if _session_state.get("initial_reset"):
    #     gr.Info("Please start a new session.")
    #     return [message, chat_history, _session_state, _context]

    # Make a POST request to the chat endpoint
    session_id = _session_state["session_id"]
    # context_dict = json.loads(_context)
    context_dict = _context
    chat_message = ChatMessage(
        agent_config=_profile_name, client_id=_client_id, reset=False, message=message, session_id=session_id, **context_dict
    )

    if not OB_PROVIDER_API_KEY:
        logger.info("No API key found, trying to use local mode for agent interactions...")
        try:
            chat_session = ChatSession.get(session_id=session_id, client_id=_client_id)
        except Exception:
            logger.info("No chat session found, resetting instead...")
            return reset(_client_id, _profile_name, chat_history, _session_state, _context)

        agent_state = {
            "frozen_agent_memory": chat_session.frozen_agent_memory,
            "frozen_agent_config": chat_session.frozen_agent_config,
        }
        gpt_agent = GptAgent.deserialize(state=agent_state, context=_context)

        response_message = gpt_agent.handle_user_message(message)

        frozen_agent_memory = gpt_agent.serialize()["frozen_agent_memory"]
        frozen_agent_config = gpt_agent.serialize()["frozen_agent_config"]

        chat_session = ChatSession(
            session_id=session_id,
            client_id=_client_id,
            frozen_agent_memory=frozen_agent_memory,
            frozen_agent_config=frozen_agent_config,
        )
        chat_session.save()
        new_context = _context

    else:
        session = _session_state["session"]
        session.headers.update(
            {
                "x-api-key": OB_PROVIDER_API_KEY,
                "origin": DEFAULT_ORIGIN,
            }
        )
        chat_message_dump = chat_message.model_dump()
        chat_message_dump["session_id"] = session_id

        response = session.post(CHAT_ENDPOINT, json=chat_message_dump)
        logger.info(f"Response: {response.json()}")
        response_message = response.json()["message"]
        _session_state["session"] = session

        response_dict = response.json()
        response_dict.pop("message")
        response_dict.pop("session_id")
        original_context = _context
        # original_context = _context
        # response_dict.update(original_context)
        original_context.update(response_dict)
        new_context = response_dict
        _session_state["last_response"] = response

    # session_state["last_response"] = response_message

    chat_history.append([message, response_message])

    # Return the response from the API
    return ["", chat_history, _session_state, new_context]


def reset(_client_id, _profile_name, chat_history, _session_state, _context):
    try:
        context_dict = _context
    except JSONDecodeError:
        context_dict = {"random_word": "dogwater"}
    # context_dict = _context
    chat_message = ChatMessage(client_id=_client_id, reset=True, agent_config=_profile_name, message="Hi", **context_dict)

    chat_message_dump = chat_message.model_dump()
    chat_message_dump.pop("message")

    response = None
    if not OB_PROVIDER_API_KEY:
        logger.info("No API key found, trying to use local mode for agent interactions...")
        # Get a new agent with the specified settings
        agent_config = AgentConfig.get(profile_name=_profile_name, client_id=_client_id)
        gpt_agent = GptAgent(agent_config=agent_config, context=context_dict)

        frozen_agent_memory = gpt_agent.serialize()["frozen_agent_memory"]
        frozen_agent_config = gpt_agent.serialize()["frozen_agent_config"]
        chat_session = ChatSession(
            client_id=_client_id,
            frozen_agent_memory=frozen_agent_memory,
            frozen_agent_config=frozen_agent_config,
        )
        session_id = chat_session.session_id
        _session_state["session_id"] = session_id
        chat_session.save()
        response_message = gpt_agent.agent_config.icebreaker
        response_dict = {}
    else:
        # Make a POST request to the reset endpoint
        session = _session_state["session"]
        headers = {
            "x-api-key": OB_PROVIDER_API_KEY,
            "origin": DEFAULT_ORIGIN,
        }
        session.headers.update(headers)
        response = session.post(url=CHAT_ENDPOINT, json=chat_message_dump)
        logger.info(f"Response: {response.json()}")
        session_id = response.cookies["Session"]
        _session_state["session_id"] = session_id
        _session_state["session"] = session
        _session_state["last_response"] = response
        response_message = response.json()["message"]
        response_dict = response.json()
        response_dict.pop("message")
        response_dict.pop("session_id")

    message = "Please wait, fetching new agent..."
    chat_history.append([message, response_message])

    # original_context = context_dict
    # original_context = _context

    # new_context = json.dumps(response_dict, indent=4)
    context_dict.update(response_dict)

    # Return the response from the API
    return ["", chat_history, _session_state, context_dict]


def save(
    _icebreaker,
    # _chat_model,
    _system_message,
    _llm,
    # _prompt_layer_tags,
    _max_iterations,
    _max_execution_time,
    _executor_temp,
    _profile_name,
    # _executor_model_type,
    # _openai_api_key,
    # _promptlayer_api_key,
    _client_id,
    # _outgoing_webhook_url,
    _record_tool_actions,
    _record_conversations,
    _tools,
):
    if _profile_name.strip() == "":
        gr.Error("This agent config must have a name (profile_name).")
        return []

    agent_config = AgentConfig(
        icebreaker=str(_icebreaker),
        # executor_chat_model=str(_chat_model),
        system_message=str(_system_message),
        llm=str(_llm),
        # prompt_layer_tags=str(_prompt_layer_tags),
        executor_max_iterations=str(_max_iterations),
        executor_max_execution_time=str(_max_execution_time),
        executor_temp=str(_executor_temp),
        profile_name=str(_profile_name),
        # executor_model_type=str(_executor_model_type),
        # openai_api_key=str(_openai_api_key),
        # promptlayer_api_key=str(_promptlayer_api_key),
        client_id=str(_client_id),
        # outgoing_webhook_url=str(_outgoing_webhook_url),
        record_tool_actions=str(_record_tool_actions),
        record_conversations=str(_record_conversations),
        tools=_tools,
    )

    # Upload the preferences to the DynamoDB database
    agent_config.save()
    logger.info(f"AgentConfig saved: {agent_config.to_json()}")
    gradio.Info("AgentConfig saved successfully.")

    # Return a success message
    return "Preferences saved successfully."


def load(_profile_name: str, _client_id: str):
    if _profile_name.strip() == "":
        raise gr.Error("client_id can't be blank.")
    # Upload the preferences to the DynamoDB database
    if not _client_id:
        _client_id = DEFAULT_CLIENT_ID
    try:
        logger.info(f"Loading agent config for profile_name: {_profile_name}, client_id: {_client_id}")
        retrieved_agent_config = AgentConfig.get(profile_name=_profile_name, client_id=_client_id)
        logger.info(f"AgentConfig loaded: {retrieved_agent_config.to_json()}")
    except Exception as e:
        new_default_agent_config = AgentConfig(client_id=_client_id, profile_name=_profile_name)
        new_default_agent_config.save()
        retrieved_agent_config = AgentConfig.get(profile_name=_profile_name, client_id=_client_id)
        logger.info(f"AgentConfig created: {retrieved_agent_config.to_json()}")

    _tools = retrieved_agent_config.tools
    supported_tools = TOOL_NAMES

    for _tool in _tools:
        if _tool not in supported_tools:
            _tools.remove(_tool)

    _agent_config = [
        str(retrieved_agent_config.icebreaker),
        # str(retrieved_agent_config.executor_chat_model),
        str(retrieved_agent_config.system_message),
        str(retrieved_agent_config.llm),
        # str(retrieved_agent_config.prompt_layer_tags),
        str(retrieved_agent_config.executor_max_iterations),
        str(retrieved_agent_config.executor_max_execution_time),
        str(retrieved_agent_config.executor_temp),
        str(retrieved_agent_config.profile_name),
        # str(retrieved_agent_config.executor_model_type),
        # str(retrieved_agent_config.openai_api_key),
        # str(retrieved_agent_config.promptlayer_api_key),
        str(retrieved_agent_config.client_id),
        # str(retrieved_agent_config.outgoing_webhook_url),
        bool(retrieved_agent_config.record_tool_actions),
        bool(retrieved_agent_config.record_conversations),
        _tools,
    ]
    llm_name = retrieved_agent_config.llm
    if llm_name in openbrain.orm.model_agent_config.FUNCTION_LANGUAGE_MODELS:
        llm_type = "function"
    elif llm_name in openbrain.orm.model_agent_config.CHAT_LANGUAGE_MODELS:
        llm_type = "chat"
    elif llm_name in openbrain.orm.model_agent_config.COMPLETION_LANGUAGE_MODELS:
        llm_type = "completion"

    else:
        raise ValueError(f"Language model {llm_name} not found in any of the language model lists")

    ret = [*_agent_config, llm_type]
    # Return a success message
    return ret


class CustomJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Decimal):
            return float(obj)
        return super(CustomJsonEncoder, self).default(obj)


def get_action_events(_events=None, _session_state=None):
    try:
        _session_id = _session_state["session_id"]
        logger.info(f"Getting Action Events: session_id {_session_id}")
    except TypeError:
        logger.info(f"Getting Action Events: failed to find session_id")

        return json.dumps({"Idle": "Start a conversation to begin monitoring for events"})
    logger.info("Getting latest action...")

    try:
        dynamodb = boto3.resource("dynamodb")
        table = dynamodb.Table(config.ACTION_TABLE_NAME)
        response = table.get_item(Key={"action_id": "latest", "session_id": _session_id})
        ret = response.get("Item", {})
        logger.info(f"Action Events: {ret}")
    except KeyError as e:
        logger.warning(f"Error finding events: {e}")
        ret = json.dumps({"exception": "Event not found for this session, perhaps one wasn't sent yet in this conversation"})
    except Exception as e:
        logger.error(f"Error finding events: {e}")
        ret = json.dumps({"exception": e.__str__()})

    if _events:
        _events = ret
    return json.dumps(ret, cls=CustomJsonEncoder, indent=4, sort_keys=True)


def get_bucket_name():
    try:
        infra_stack_name = config.INFRA_STACK_NAME
        # Get tablename from outputs of INFRA_STACK
        cfn = boto3.client("cloudformation")
        response = cfn.describe_stacks(StackName=infra_stack_name)
        outputs = response["Stacks"][0]["Outputs"]
        for output in outputs:
            if output["OutputKey"] == "ObBucketName":
                bucket = output["OutputValue"]
                break
        else:
            raise Exception("Bucket name not found in outputs")
        return bucket
    except Exception as e:
        raise e


def get_bottom_text(_session_state=None, _client_id=None, _profile_name=None):
    # Get recorded conversation link if it exists
    logger.info("Getting bottom text...")
    logger.info(f"Session state: {_session_state}")
    logger.info(f"Client ID: {_client_id}")
    logger.info(f"Profile Name: {_profile_name}")

    try:
        bucket_name = get_bucket_name()
        _session_id = _session_state.get("session_id").lower()
        dl_url = f"https://{bucket_name}.s3.amazonaws.com/conversations/{_profile_name}/{_client_id}/{_session_id}.json"
        link_text_md = f"| [Download Session Data]({dl_url}) "

    except (NoCredentialsError, AttributeError) as e:
        logger.error(f"Error: {e}")
        link_text_md = ""
        _session_id = "`no-session`"

    try:
        xray_trace_id = _session_state["last_response"].headers["x-amzn-trace-id"]
        xray_trace_id = re.sub(r";.*", "", xray_trace_id.replace("Root=", ""))

        xray_trace_id_md = f"| [X-Ray](https://console.aws.amazon.com/xray/home?region=us-east-1#/traces/{xray_trace_id}) "
    except (TypeError, AttributeError):
        xray_trace_id_md = ""

    if config.OB_MODE == Defaults.OB_MODE_LOCAL.value:
        orm_mode = "Ephemeral"
    else:
        orm_mode = "Persistent"
    orm_mode_md = f"| ORM: `{orm_mode}` "

    api = f"{CHAT_ENDPOINT}" if OB_PROVIDER_API_KEY else "LOCAL"
    api_md = f"| API: `{api}` "

    infra_stack_name = os.environ["INFRA_STACK_NAME"]
    infra_stack_name_md = f"| Stack: `{infra_stack_name}` "

    user_api_key_link = f"{PORTAL_URL}/dashboard"
    user_api_key_link_md = f"| [API Key]({user_api_key_link}) "

    ob_version = openbrain.__version__
    ob_tuner_version = OB_TUNER_VERSION

    versions_md = f"| Versions: OB=`{ob_version}` OB-Tuner=`{ob_tuner_version}`"

    formatted_text = (
        xray_trace_id_md
        + user_api_key_link_md
        + link_text_md
        + versions_md
        + infra_stack_name_md
        + orm_mode_md
        + api_md
        + f"| Session: `{_session_id}` |"
    )
    return formatted_text


def get_gpt_agent_logs():
    if OB_PROVIDER_API_KEY:
        return "This function is only available when running on-tuner with local openbrain code."
    else:
        agent_logger = openbrain.util.get_logger()
        for stream in agent_logger.handlers:
            if isinstance(stream.stream, StringIO):
                return stream.stream.getvalue().replace("\\n", "\n")


def get_context(
        _location_id,
        _calendar_id,
        _contact_id,
        _context_firstName,
        _context_lastName,
        _context_name,
        _context_dateOfBirth,
        _context_phone,
        _context_email,
        _context_address1,
        _context_city,
        _context_state,
        _context_country,
        _context_postalCode,
        _context_companyName,
        _context_website,
        _context_medications,
        _context_random_word,
):
    context_json = {
        "locationId": _location_id,
        "calendarId": _calendar_id,
        "contactId": _contact_id,
        "firstName": _context_firstName,
        "lastName": _context_lastName,
        "name": _context_name,
        "dateOfBirth": _context_dateOfBirth,
        "phone": _context_phone,
        "email": _context_email,
        "address1": _context_address1,
        "city": _context_city,
        "state": _context_state,
        "country": _context_country,
        "postalCode": _context_postalCode,
        "companyName": _context_companyName,
        "website": _context_website,
        "medications": _context_medications,
        "random_word": _context_random_word,
    }
    ret_dict = {}
    for key in context_json:
        if context_json[key]:
            ret_dict[key] = context_json[key]
    return json.dumps(ret_dict, indent=2)


def save_context(
        _location_id,
        _calendar_id,
        _contact_id,
        _context_firstName,
        _context_lastName,
        _context_name,
        _context_dateOfBirth,
        _context_phone,
        _context_email,
        _context_address1,
        _context_city,
        _context_state,
        _context_country,
        _context_postalCode,
        _context_companyName,
        _context_website,
        _context_medications,
        _context_random_word,
):
    context = get_context(
        _location_id,
        _calendar_id,
        _contact_id,
        _context_firstName,
        _context_lastName,
        _context_name,
        _context_dateOfBirth,
        _context_phone,
        _context_email,
        _context_address1,
        _context_city,
        _context_state,
        _context_country,
        _context_postalCode,
        _context_companyName,
        _context_website,
        _context_medications,
        _context_random_word,
    )
    raise gr.Error("Saving not yet implemented 💥!", duration=5)


def enable_chat_button():
    return gr.Button("Chat", variant="primary", elem_id="chat_button", elem_classes=["chatbot"], interactive=True)


def enable_or_disable_save(_client_id, _profile_name):
    _registered_client_ids = get_registered_client_ids()
    # Remove the default client_id
    # available_client_ids.append(_client_id)

    # If client_id is public, allow saving except to the default profile
    if _client_id == DEFAULT_CLIENT_ID:
        if _profile_name == DEFAULT_PROFILE_NAME:
            return gr.Button(value="Save Agent", variant="secondary", elem_id="save_agent_config_button", elem_classes=["agent_config"], interactive=False)
        else:
            return gr.Button(value="Save Agent", variant="secondary", elem_id="save_agent_config_button", elem_classes=["agent_config"], interactive=True)


    elif _client_id in _registered_client_ids:
        return gr.Button(value="Save Agent", variant="secondary", elem_id="save_agent_config_button", elem_classes=["agent_config"], interactive=False)
    else:
        return gr.Button(value="Save Agent", variant="secondary", elem_id="save_agent_config_button",
                                      elem_classes=["agent_config"], interactive=True)

# def generate_context(_username):
#     client = Client.get(email=_username)
#     leadmo_location_id = client.leadmo_location_id or "UNKNOWN"
#     # leadmo_contact_id_phone = client.leadmo_contact_id_phone or "UNKNOWN"
#     random_word = faker.Faker().word()
#     context = {
#         "locationId": leadmo_location_id,
#         "random_word": random_word,
#     }
#     result = context.update(client.model_dump())
#     return result




import requests

from ob_tuner.functions.button_logic import *
from ob_tuner.util import *

WELCOME_MESSAGE = f"""
# {CUSTOMER_NAME} AI Workshop

## Introduction
Welcome to the {CUSTOMER_NAME} AI Tuning app! 
 
## Main Features
- Manage your account's API keys for AI tools that use external APIs.
- Create agent configuration profiles with custom values for:
    - Save and load named agent configuration profiles to/from your account.
    - Interact with your named agents, or community (shared) named agents.
    - Interact with {CUSTOMER_NAME} with context to simulate the AI interacting with the Lead Momentum SMS Workflow.
    - LLM Type - supported types include
        - function
        - chat
        - completion
    - Tools - List of tools the agent can use. To see tool descriptions, navigate to the Tools tab
    - System Message - Message to display when the agent is first started.
    - Icebreaker - Message to display when the agent is first started.
    - LLM - name of the LLM to use (e.g. gpt-3, gpt-4o, etc.)
    - Temperature - Higher values result in spicier agents with less reliable tool use.
    - Max Execution Time - Maximum time allowed for agent to execute a tool.
    - Max Iterations - Maximum number of iterations allowed for agent to execute a tool.
    - Record Chat - Record chat messages for this agent for later analysis.
    - Record Tool Actions - Record tool actions for this agent for later analysis.
"""

with gr.Blocks(analytics_enabled=False, theme="freddyaboulton/dracula_revamped") as landing_page:
    # Read the README file content
    with gr.Row():
        with gr.Column(scale=10):
            if NOAUTH_DEMO_PAGE:
                gr.Markdown(
                    value="# Welcome to OpenBra.in!\n Please enjoy the live demo. Load an agent profile, and start chatting with the AI. Explore the tools. If you are interested in trying OpenBra.in, please sign up for an account. Once signed up you can create and save your own agent profiles.\nDeployed agents can be used in your own applications via the OpenBra.in API, the following website is using the `openbrain` `promoter` agent.\n\n[OpenBra.in Demo Website](https://www.openbra.in/) (allow 15 seconds for the agent to initialize).")
            else:
                gr.Markdown(value="# Welcome!\n Please sign up or login. ")

        with gr.Column(scale=1) as login_column:
            gr.Button("Login", link="/login", variant="primary", size="sm")
            gr.Button("Signup", link="/signup", variant="secondary", size="sm")

    if NOAUTH_DEMO_PAGE:

        with gr.Tab("Demo"):
            session_state = gr.State(
                value={"session_id": "", "session": requests.Session(), "agent": None, "last_response": None,
                       "username": ""})

            with gr.Tab("Selected Agent Configuration") as submit_tab:
                with gr.Row() as submit_row:
                    with gr.Column(scale=3) as key_text_column:
                        with gr.Row() as key_text_row:
                            client_id = gr.Dropdown(
                                label="Client ID",
                                info="Develop your own AI agents or use a community agent.",
                                choices=["openbrain"],
                                value="openbrain",
                            )

                            profile_name = gr.Dropdown(
                                allow_custom_value=True,
                                label="Profile Name",
                                info="The name of your AgentConfig. Defaults to 'default'",
                                choices=[str(DEFAULT_PROFILE_NAME), 'promoter'],
                                value='promoter',
                                interactive=False,
                            )

                    with gr.Column(scale=1) as submit_column:
                        load_agent_profile_button = gr.Button(value="Reload Agent", variant="secondary")
                        save_agent_profile_button = gr.Button(value="Save Agent", variant="secondary", render=True, interactive=False)

            with gr.Accordion("AI Agent Configuration and Tuning", elem_classes="accordion", visible=is_settings_set(),
                              open=False) as prompts_box:
                with gr.Tab("LLM Parameters") as tuning_tab:
                    gr.Markdown(
                        "Changes to these settings are used to set up a conversation using the Reset button and will not "
                        "be reflected until the next 'Reset'"
                    )

                    with gr.Row() as preferences_row1:
                        options = model_agent_config.EXECUTOR_MODEL_TYPES
                        default_llm_types = ["function"]
                        llm_types = gr.CheckboxGroup(
                            choices=options,
                            label="LLM Types",
                            info="List only the types of agents you are interested in.",
                            value=default_llm_types,
                        )
                        original_choices = model_agent_config.FUNCTION_LANGUAGE_MODELS
                        # original_value = openbrain.orm.model_agent_config.FUNCTION_LANGUAGE_MODELS[0]
                        llm = gr.Dropdown(
                            choices=original_choices,
                            # value=original_value,
                            label="LLM",
                            info="The language model to use for completion",
                        )

                        with gr.Column() as extra_options_column:
                            record_tool_actions = gr.Checkbox(
                                label="Record Tool Actions", info="Record tool actions (use 'Actions' box).",
                                interactive=False
                            )
                            record_conversations = gr.Checkbox(label="Record Conversations",
                                                               info="Record conversations.",
                                                               interactive=False)

                        # executor_completion_model = gr.Dropdown( choices=["text-davinci-003", "text-davinci-002",
                        # "text-curie-001", "text-babbage-001", "text-ada-001"], label="Executor Completion Model",
                        # info="This is the model used when Executor Model Type is set to 'completion'" )

                    with gr.Row() as preferences_row2:
                        executor_temp = gr.Slider(
                            minimum=0,
                            maximum=2,
                            label="Temperature",
                            step=0.1,
                            info="Higher temperature for 'spicy' agents.",
                        )
                        max_execution_time = gr.Slider(
                            minimum=0,
                            maximum=120,
                            label="Max Execution Time",
                            step=0.5,
                            info="Maximum agent response time before termination.",
                        )
                        max_iterations = gr.Number(
                            label="Max Iterations",
                            info="Number of steps an agent can take for a response before termination.",
                        )

                with gr.Tab("Tools") as preferences_row3:
                    tools = gr.CheckboxGroup(choices=TOOL_NAMES, value=[], label="Tools",
                                             info="Select tools to enable for the agent", interactive=False)

                    gr.Markdown(
                        value="Tool descriptions as presented to the AI. Confusing text here could lead to inconsistent use of tools."
                    )
                    _tools = discovered_tools
                    with gr.Column() as tool_accordion:
                        for _tool in _tools:
                            with gr.Tab(_tool.name):
                                gr.Markdown(value=get_tool_description(_tool.name))

                with gr.Tab("System Message") as long_text_row1:
                    system_message = gr.TextArea(
                        lines=10,
                        label="System Message",
                        placeholder="Enter your system message here",
                        info="The System Message. This message is a part of every context with "
                             "ChatGPT, and is therefore the most influential, and expensive place to "
                             "add text",
                        show_label=False,
                    )

                with gr.Tab("Icebreaker") as long_text_row2:
                    icebreaker = gr.TextArea(
                        lines=10,
                        label="Icebreaker",
                        placeholder="Enter your icebreaker here",
                        show_label=False,
                        info="The first message to be sent to the user.",
                    )

                with gr.Tab("Documentation") as help_tab:
                    gr.Markdown(value=get_help_text())

            with gr.Accordion("Interact with Agent") as chat_accordian:
                with gr.Row() as chat_row:
                    with gr.Column(scale=2) as chat_container:
                        with gr.Column(scale=2) as chat_column:
                            chatbot = gr.Chatbot(
                                show_share_button=True,
                                show_copy_button=True,
                                # avatar=(user_avatar_path, ai_avatar_path),
                                likeable=True,
                                # layout="panel",
                                layout="bubble",

                            )

                            msg = gr.Textbox()

                    with gr.Column(scale=1) as context_container:
                        initial_value_str = json.loads(json.dumps({
                            "first_name": "John",
                            "last_name": "Doe",
                            "locationId": "loc-123456",
                            "calendarId": "cal-123456",
                            "random_word": "Rambutan"
                        }))

                        gr.Markdown("OpenBra.in's API accepts arbitrary key/value pairs. Tools like book appointment require parameters like `calendarId=23nio2n3nf2oi32`, which cause halucinations. This gets around that by creating tools with values sent in by the API call.")
                        context = gr.JSON(
                            label="Context",
                            value=initial_value_str
                        )
                        landing_page.load(lambda: gr.JSON(label="Context", value=initial_value_str), outputs=[context])

                        chat_button = gr.Button("Chat", variant="primary", interactive=False)
                        reset_agent = gr.Button("Reset", variant="secondary")

                        msg.submit(
                            chat,
                            [msg, chatbot, profile_name, session_state, client_id, context],
                            [msg, chatbot, session_state, context],
                        )

            with gr.Row() as bottom_text_row:
                logout_button = gr.Button("Logout", link="/logout", variant="secondary", size="sm", scale=1, render=False)
                with gr.Column(scale=10):
                    bottom_text = gr.Markdown(value=get_bottom_text(), rtl=True)

            preferences = [
                icebreaker,
                system_message,
                llm,
                max_iterations,
                max_execution_time,
                executor_temp,
                profile_name,
                client_id,
                record_tool_actions,
                record_conversations,
                tools,
                llm_types,
            ]

            # Refresh Buttons

            save_agent_profile_button.click(
                fn=lambda: gr.Warning("Saving disabled in the demo, please sign up for an account."))
            load_agent_profile_button.click(load, inputs=[profile_name, client_id], outputs=preferences)

            profile_name.change(load, inputs=[profile_name, client_id], outputs=preferences)
            landing_page.load(load, inputs=[profile_name, client_id], outputs=preferences)

            # Chat Button
            chat_button.click(
                fn=chat,
                inputs=[msg, chatbot, profile_name, session_state, client_id, context],
                outputs=[msg, chatbot, session_state, context],
            )
            dummy_text = gr.Textbox(visible=False)
            chat_button.click(get_aws_cloudwatch_logs, inputs=[session_state], outputs=[dummy_text])

            # Chat Reset Button
            reset_agent.click(
                fn=reset,
                inputs=[client_id, profile_name, chatbot, session_state, context],
                outputs=[msg, chatbot, session_state, context],
            )
            # reset_agent.click(get_bottom_text, inputs=[session_state, profile_name, client_id], outputs=[context])
            reset_agent.click(get_aws_cloudwatch_logs, inputs=[session_state], outputs=[dummy_text])
            reset_agent.click(get_action_events, inputs=[dummy_text, session_state], outputs=[dummy_text])
            reset_agent.click(enable_chat_button, outputs=[chat_button])

            # On Change Events
            llm_types.change(get_llm_choices, inputs=[llm_types], outputs=[llm])
            # client_id.change(update_available_profile_names, inputs=[client_id], outputs=[profile_name])

    with gr.Tab(f"Welcome to {CUSTOMER_NAME}"):
        gr.Markdown(WELCOME_MESSAGE)


if __name__ == "__main__":
    if os.getenv("GRADIO_PORT"):
        gradio_host = os.getenv("GRADIO_HOST", '0.0.0.0')
        gradio_port = int(os.getenv("GRADIO_PORT", 7861))
        landing_page.launch(
            debug=True,
            share=False,
            server_name=gradio_host,
            server_port=gradio_port,
        )



import datetime
import json
import os
import pathlib
import time
from logging.handlers import RotatingFileHandler
import boto3
import faker
import gradio as gr
import logging
from io import StringIO

import requests
from dotenv import load_dotenv, find_dotenv
from importlib.metadata import version
from boto3.dynamodb.conditions import Attr
# Use find_dotenv to locate the .env file
dotenv_path = find_dotenv(usecwd=True)
if dotenv_path:
    load_dotenv(dotenv_path)

os.environ["INFRA_STACK_NAME"] = os.environ.get("INFRA_STACK_NAME", "LOCAL")
# Unfortunate... fix later
from openbrain.orm.model_client import Client

import openbrain.orm
import openbrain.util
from openbrain.orm.model_agent_config import AgentConfig
from openbrain.orm.model_common_base import InMemoryDb
from openbrain.util import config, Defaults
from openbrain.tools import Toolbox


# Logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# Add a rotating log handler
formatter = logging.Formatter('%(filename)-20s:%(lineno)-4d %(levelname)-8s %(message)s')

handler = RotatingFileHandler('app.log', maxBytes=10000, backupCount=5)
handler.setLevel(logging.DEBUG)
handler.setFormatter(formatter)
logger.addHandler(handler)

# Add a stringio handler
log_stream = StringIO()
string_handler = logging.StreamHandler(log_stream)
string_handler.setLevel(logging.DEBUG)
string_handler.setFormatter(formatter)
logger.addHandler(string_handler)
ADMIN_USER = os.environ.get("ADMIN_USER")
if os.environ.get("OB_MODE", "LOCAL") == "LOCAL" and ADMIN_USER:
    logger.info(f"Creating admin user {ADMIN_USER}")
    try:
        ADMIN_LEADMO_LOCATION_ID = os.getenv("ADMIN_LEADMO_LOCATION_ID")
        ADMIN_LEADMO_API_KEY = os.getenv("ADMIN_LEADMO_API_KEY")
        ADMIN_LEADMO_CALENDAR_ID = os.getenv("ADMIN_LEADMO_CALENDAR_ID")
        admin_user = Client(email=ADMIN_USER, leadmo_api_key=ADMIN_LEADMO_API_KEY, leadmo_location_id=ADMIN_LEADMO_LOCATION_ID, leadmo_calendar_id=ADMIN_LEADMO_CALENDAR_ID)
        admin_user.save()
        admin_default_agent_config = AgentConfig(client_id=ADMIN_USER, profile_name=Defaults.DEFAULT_PROFILE_NAME.value)
        admin_default_agent_config.save()
    except Exception as e:
        admin_user = Client(email=ADMIN_USER)
        admin_user.save()
        logger.info(f"Created admin user {ADMIN_USER}")

INFRA_STACK_NAME = os.environ.get("INFRA_STACK_NAME")
OB_MODE = config.OB_MODE
CHAT_ENDPOINT = os.environ.get("OB_API_URL", "") + "/chat"
DEFAULT_ORIGIN = os.environ.get("DEFAULT_ORIGIN", "https://localhost:5173")
OB_PROVIDER_API_KEY = os.environ.get("OB_PROVIDER_API_KEY", "")
DEFAULT_CLIENT_ID: str = openbrain.util.Defaults.DEFAULT_CLIENT_ID.value
DEFAULT_PROFILE_NAME = Defaults.DEFAULT_PROFILE_NAME.value
PORTAL_URL = os.getenv("PORTAL_URL")
COGNITO_DOMAIN = os.getenv("COGNITO_DOMAIN")
CLIENT_ID = os.getenv("CLIENT_ID")
# CALLBACK_URL = os.getenv("CALLBACK_URL")
DEPLOYMENT_URL = os.getenv("DEPLOYMENT_URL")
CLIENT_SECRET = os.getenv("CLIENT_SECRET")
GUEST_CLIENT_ID = "guest@openbra.in"
discovered_tools = Toolbox.discovered_tools
model_agent_config = openbrain.orm.model_agent_config
OB_TUNER_VERSION = version("ob-tuner")
CUSTOMER_NAME = os.getenv("CUSTOMER_NAME")


NOAUTH_DEMO_PAGE_STR = os.getenv("NOAUTH_DEMO_PAGE", "False")
LEADMO_INTEGRATION_STR = os.getenv("LEADMO_INTEGRATION", "False")
LLS_INTEGRATION_STR = os.getenv("LLS_INTEGRATION", "False")
NOAUTH_DEMO_PAGE = NOAUTH_DEMO_PAGE_STR.casefold() == "true"
LEADMO_INTEGRATION = LEADMO_INTEGRATION_STR.casefold() == "true"
LLS_INTEGRATION = LLS_INTEGRATION_STR.casefold() == "true"

logger.info(f"************************************ GRADIO CONFIG ************************************")
logger.info(f"DEFAULT_ORIGIN: {DEFAULT_ORIGIN}")
logger.info("OB_API_URL: " + os.environ.get("OB_API_URL", ""))
logger.info(f"({type(NOAUTH_DEMO_PAGE)}): {NOAUTH_DEMO_PAGE=}")
logger.info(f"({type(LEADMO_INTEGRATION)}): {LEADMO_INTEGRATION=}")
logger.info(f"({type(LLS_INTEGRATION)}): {LLS_INTEGRATION=}")
logger.info(f"{CUSTOMER_NAME=}")
logger.info(f"{LEADMO_INTEGRATION=}")
logger.info(f"{NOAUTH_DEMO_PAGE=}")
logger.info(f"{LLS_INTEGRATION=}")
logger.info(f"{OB_TUNER_VERSION=}")

logger.info(f"************************************ AUTH ************************************")
logger.info(f'COGNITO_DOMAIN: {COGNITO_DOMAIN}')
logger.info(f'CLIENT_ID: {CLIENT_ID}')
# logger.info(f'CALLBACK_URL: {CALLBACK_URL}')
logger.info(f'{PORTAL_URL=}')
logger.info(f'DEPLOYMENT_URL: {DEPLOYMENT_URL}')
logger.info("DEFAULT_ORIGIN: " + os.environ.get("DEFAULT_ORIGIN", "https://localhost:5173"))

logger.info(f"************************************ OPENBRAIN CONFIG ************************************")
logger.info(f"OB_MODE: {OB_MODE}")
logger.info(f"SESSION_TABLE_NAME: {config.SESSION_TABLE_NAME}")
logger.info(f"AGENT_CONFIG_TABLE_NAME: {config.AGENT_CONFIG_TABLE_NAME}")
logger.info(f"ACTION_TABLE_NAME: {config.ACTION_TABLE_NAME}")
logger.info(f"{INFRA_STACK_NAME=}")

logger.info(f"************************************ SECRETS ************************************")
OBFUSCATED_CLIENT_SECRET = CLIENT_SECRET[:2] + "*" * (len(CLIENT_SECRET) - 2)
OBFUSCATED_OB_PROVIDER_API_KEY = os.environ.get("GRADIO_OB_PROVIDER_API_KEY", "")[:2] + "*" * (len(os.environ.get("GRADIO_OB_PROVIDER_API_KEY", "")) - 2)
logger.info(f'CLIENT_SECRET: {OBFUSCATED_CLIENT_SECRET}')
logger.info("OB_PROVIDER_API_KEY: " + OBFUSCATED_OB_PROVIDER_API_KEY)

string_handler.flush()
tool_names = [tool.name for tool in Toolbox.discovered_tools if not tool.name.startswith("leadmo_")]
leadmo_tool_names = [tool.name for tool in Toolbox.discovered_tools if tool.name.startswith("leadmo_")]

tool_names.sort()
leadmo_tool_names.sort()
TOOL_NAMES = tool_names + leadmo_tool_names if LEADMO_INTEGRATION else tool_names
logger.info(f"Tool names: {TOOL_NAMES}")

REGISTERED_CLIENT_IDS = {
    "WoxomAI": [DEFAULT_CLIENT_ID, "woxom", "leadmo"],
    "OpenBra.in": [DEFAULT_CLIENT_ID, 'openbrain'],
    "OBTest": [DEFAULT_CLIENT_ID, 'openbrain', 'woxom', 'leadmo'],
}
REGISTERED_THEMES = {
    "WoxomAI": ["light", "dark"],
    "OpenBra.in": ["light", "dark"],
    "OBTest": ["light", "dark"],
}



def get_debug_text(_debug_text=None) -> str:
    try:
        ret = log_stream.getvalue()
    except Exception as e:
        ret = e.__str__()

    if _debug_text:
        _debug_text = ret

    return ret


def get_aws_xray_trace_summaries(id=None):
    """Get x-ray logs from AWS"""
    client = boto3.client("xray")
    this_year = datetime.datetime.now().year
    this_month = datetime.datetime.now().month
    this_day = datetime.datetime.now().day
    yesterday = (datetime.datetime.now() - datetime.timedelta(days=1)).day

    if id:
        response = client.get_trace_summaries(
            StartTime=datetime.datetime(this_year, this_month, yesterday),
            EndTime=datetime.datetime(this_year, this_month, this_day),
            Sampling=False,
            FilterExpression=f"traceId = {id}",
        )
    else:
        response = client.get_trace_summaries(
            StartTime=datetime.datetime(this_year, this_month, yesterday),
            EndTime=datetime.datetime(this_year, this_month, this_day),
            Sampling=False,
        )

    return response


def is_settings_set():
    return True


def get_tool_description(tool_name):
    for tool in Toolbox.discovered_tools:
        if tool_name == tool.name:
            tool_instance = tool.tool()
            tool_description = tool_instance.description
            fields = tool_instance.args_schema.model_fields
            args_string = ""
            if not fields:
                args_string = "'No args'"
            for field in fields:
                field_str = fields[field].__str__()
                args_string += f"{field}: {field_str}\n"
            tool_description = f"""#### Description
{tool_description}

#### Args
```python
{args_string}
```"""
            return tool_description


def get_available_tool_descriptions():
    tool_descriptions = []

    for tool in Toolbox.discovered_tools:
        tool_name = tool.name
        tool_instance = tool.tool()
        tool_description = tool_instance.description
        fields = tool_instance.args_schema.model_fields
        args_string = ""
        if not fields:
            args_string = "'No args'"
        for field in fields:
            field_str = fields[field].__str__()
            args_string += f"{field}: {field_str}\n"
        tool_description = f"""
## Tool: {tool_name}

#### Description
{tool_description}

#### Args
```python
{args_string}
```
---"""

        tool_descriptions.append(tool_description)

    tool_descriptions_string = "\n".join(tool_descriptions)
    return tool_descriptions_string


def get_available_profile_names(_client_id) -> list:
    logger.info(f"Getting available profile names for {_client_id}")
    # logger.warning("get_available_profile_names() is not implemented")
    # Get AgentConfig table

    if OB_MODE == Defaults.OB_MODE_LOCAL.value:
        try:
            logger.info(f"Checking InMemoryDb for available profile names...")
            lst = list(InMemoryDb.instance[config.AGENT_CONFIG_TABLE_NAME][_client_id].keys())
            return lst
        except Exception:
            logger.warning(f"Not found, initializing default profile name...")
            _client_id = _client_id or GUEST_CLIENT_ID
            default_config = AgentConfig(client_id=_client_id, profile_name=DEFAULT_PROFILE_NAME)
            logger.info(f"Saving default profile name: {default_config}")
            default_config.save()
            logger.info(f"Saved default profile name: {default_config}")
            lst = list(InMemoryDb.instance['agent_config_table'][_client_id].keys())
            logger.info(f"Available profile names: {lst}")
            return lst
    else:
        table = boto3.resource("dynamodb").Table(config.AGENT_CONFIG_TABLE_NAME)
        # get all items in the table
        try:
            items = table.scan()["Items"]
            available_profile_names = [item["profile_name"] for item in items if item["client_id"] == _client_id]
            # response = table.query(
            #     KeyConditionExpression=boto3.dynamodb.conditions.Key('client_id').eq(_client_id)
            # )
            # available_profile_names = [profile["profile_name"] for profile in response]

        except Exception as e:
            logger.error(f"Error getting available profile names: {e}")
            available_profile_names = []

        return available_profile_names


def update_available_profile_names(client_id):
    if not client_id:
        return gr.Dropdown(choices=[DEFAULT_PROFILE_NAME], value=DEFAULT_PROFILE_NAME)

    available_profile_names = get_available_profile_names(client_id)
    # available_profile_names = [profile["profile_name"] for profile in available_profiles]
    try:
        selected_profile = DEFAULT_PROFILE_NAME
    except IndexError:
        default_config = AgentConfig(client_id=client_id, profile_name=DEFAULT_PROFILE_NAME)
        default_config.save()
        selected_profile = DEFAULT_PROFILE_NAME
        available_profile_names = [DEFAULT_PROFILE_NAME]
    return gr.Dropdown(choices=available_profile_names, value=selected_profile)


def get_llm_choices(llm_types=None):
    """Get the available LLM choices based on the selected types"""
    if not llm_types:
        llm_types = ["function"]
    available_llms = []
    known_llm_types = openbrain.orm.model_agent_config.EXECUTOR_MODEL_TYPES
    for llm_type in llm_types:
        if llm_type == "function":
            available_llms += openbrain.orm.model_agent_config.FUNCTION_LANGUAGE_MODELS
        elif llm_type == "chat":
            available_llms += openbrain.orm.model_agent_config.CHAT_LANGUAGE_MODELS
        elif llm_type == "completion":
            available_llms += openbrain.orm.model_agent_config.COMPLETION_LANGUAGE_MODELS
        else:
            logger.error(f"Unknown LLM type: {llm_type}, must be one of {known_llm_types}")
            continue
    return gr.Dropdown(choices=available_llms)


def greet(request: gr.Request):
    try:
        return f"Welcome to Gradio, {request.username}"
    except Exception:
        return "OH NO!"


def initialize_username(request: gr.Request):
    _username = request.username or GUEST_CLIENT_ID
    logger.info(f"Initializing username: {_username}")
    registered_client_ids = get_registered_client_ids()
    # Copying the dict so we don't update it
    registered_client_ids_copy = registered_client_ids.copy()
    registered_client_ids_copy.append(_username)
    client_id_dropdown = gr.Dropdown(choices=registered_client_ids_copy, value=_username)

    try:
        AgentConfig.get(client_id=_username, profile_name=DEFAULT_PROFILE_NAME)
    except Exception as e:
        logger.info(f"No default AgentConfig found for {_username}, creating default agent config for {_username}")
        new_default_agent_config = AgentConfig(client_id=_username, profile_name=DEFAULT_PROFILE_NAME)
        new_default_agent_config.save()
    return [_username, client_id_dropdown]
    # return [_username]

def update_client_id(_username, _session_state):
    _username = _session_state["username"]
    _username_from_session_state = _session_state["username"]

    if _username != _username_from_session_state:
        _session_state["username"] = _username
        logger.info(f"Updating session state (MISMATCH): {_username=}")

    logger.info(f"Updating client_id: {_username=}")
    logger.info(f"Updating client_id: {DEFAULT_CLIENT_ID=}")
    registered_client_ids = get_registered_client_ids()
    registered_client_ids_copy = registered_client_ids.copy()
    registered_client_ids_copy.append(_username)

    return gr.Dropdown(choices=registered_client_ids_copy, value=_username)

# def get_session_username(_session_state=None):
#     username = _session_state["username"]
#     _choices = [DEFAULT_CLIENT_ID, username]
#     return gr.Dropdown(choices=_choices, value=username)


def get_help_text() -> str:
    current_dir = pathlib.Path(__file__).parent
    help_text_path = current_dir / "resources" / "help_text.md"
    with open(help_text_path, "r", encoding="utf8") as file:
        # Read line in UTF-8 format
        help_text = file.readlines()
    return ''.join(help_text)

def get_registered_client_ids(customer_name=CUSTOMER_NAME):
    registered_client_ids = REGISTERED_CLIENT_IDS.get(customer_name, [DEFAULT_CLIENT_ID])
    return registered_client_ids








# EXAMPLE_CONTEXT = """
# {
#     "locationId": "LEADMOMENTUMLOCATIONID",
#     "calendarId": "LEADMOMENTUMCALENDARID",
#     "contactId": "CONTACTID",
#     "random_word": "spatula",
#     "firstName": "Cary",
#     "lastName": "Nutzington",
#     "name": "Cary Nutzington",
#     "dateOfBirth": "1970-04-01",
#     "phone": "+16198675309",
#     "email": "example@email.com",
#     "address1": "1234 5th St N",
#     "city": "San Diego",
#     "state": "CA",
#     "country": "US",
#     "postalCode": "92108",
#     "companyName": "Augmenting Integrations",
#     "website": "openbra.in",
#     "medications": "tylonol"
# }
# """.strip()

EXAMPLE_CONTEXT = {
    "locationId": "LEADMOMENTUMLOCATIONID",
    "calendarId": "LEADMOMENTUMCALENDARID",
    "contactId": "CONTACTID",
    "random_word": "spatula",
}
def set_client_id(_username):
    registered_client_ids = get_registered_client_ids()
    # Copying the dict so we don't update it
    registered_client_ids_copy = registered_client_ids.copy()
    registered_client_ids_copy.append(_username)
    client_id = gr.Dropdown(
        label="Client ID",
        filterable=True,
        info="Develop your own AI agents or use a community agent.",
        choices=registered_client_ids_copy,
        value=_username,
        elem_id="client_id",
        elem_classes=["agent_config"],
    )

    return client_id


def fetch_user_from_leadmo(context, username, leadmo_api_key, leadmo_location_id, leadmo_calendar_id, first_name, last_name, date_of_birth, phone, address1, address2, city, state, country, postal_code, website):
    """Fetch user from Lead Momentum. If that fails, create a new user."""
    user = Client.get(email=username)

    try:
        leadmo_api_key = user.leadmo_api_key
    except Exception as e:
        gr.Error(f"Please save your Lead Momentum Location ID and API key to your profile.")
        return context

    # First try to fetch with the phone number, and if that fails, try with the email address
    LEADMO_API_V1_CONTACTS_URL = 'https://rest.gohighlevel.com/v1/contacts/lookup'

    url_encoded_phone = phone.replace("+", "%2B")
    url_encoded_email = username.replace("@", "%40")

    phone_url = LEADMO_API_V1_CONTACTS_URL + f'?phone={url_encoded_phone}'
    email_url = LEADMO_API_V1_CONTACTS_URL + f'?email={url_encoded_email}'
    response = None

    headers = {
        "Content-Type": "application/json",
        "Origin": DEFAULT_ORIGIN,
        "Authorization": f'Bearer {leadmo_api_key}'
    }
    response = requests.get(url=phone_url, headers=headers)
    if not (200 <= response.status_code < 300):
        logger.error(f"Failed to fetch user from Lead Momentum with phone number.")
        response = None

    if not response:
        response = requests.get(url=email_url, headers=headers)
    if not (200 <= response.status_code < 300):
        logger.error(f"Failed to fetch user from Lead Momentum with email address.")
        response = None

    if not response:
    # Create a contact
        url = LEADMO_API_V1_CONTACTS_URL
        headers = {
            "Content-Type": "application/json",
            "Origin": DEFAULT_ORIGIN,
            "Authorization ": f'Bearer {leadmo_api_key}'
        }

        data = {
            "email": username,
            "phone": phone,
            "firstName": first_name,
            "lastName": last_name,
            "name": f"{first_name} {last_name}",
            "dateOfBirth": date_of_birth,
            "address1": address1,
            "city": city,
            "state": state,
            "country": country,
            "postalCode": postal_code,
            "companyName": "Augmenting Integrations",
            "website": website,
            "tags": ["woxom_ai"]
        }

        response = requests.post(url=url, headers=headers, data=json.dumps(data))
        logger.info(f"Successfully created user in Lead Momentum.")

    if not (200 <= response.status_code < 300):
        logger.error(f"Failed to create user in Lead Momentum.")
        return context


    context = response.json()["contacts"][0]
    context = {
        "locationId": leadmo_location_id,
        "calendarId": leadmo_calendar_id,
        "contactId": context.get("id", ""),
        "firstName": context.get("firstName", ""),
        "lastName": context.get("lastName", ""),
        "name": context.get("name", ""),
        "dateOfBirth": context.get("dateOfBirth", ""),
        "phone": context.get("phone", ""),
        "email": context.get("email", ""),
        "address1": context.get("address1", ""),
        "city": context.get("city", ""),
        "state": context.get("state", ""),
        "country": context.get("country", ""),
        "postalCode": context.get("postalCode", ""),
        "companyName": context.get("companyName", ""),
        "website": context.get("website", ""),
    }
    return context    # context2, leadmo_contact_id_phone, leadmo_location_id, lls_api_key


# username.change(fill_context, inputs=[context, username], outputs=user_leadmo_details + user_personal_details)
def fill_context(username, leadmo_api_key, leadmo_location_id, leadmo_calendar_id, first_name, last_name, date_of_birth, phone, address1, address2, city, state, country, postal_code, website):
    """Fill the context with the user's details from Lead Momentum"""
    random_word = faker.Faker().word()
    context = {
        "locationId": leadmo_location_id,
        "calendarId": leadmo_calendar_id,
        "contactId": "",
        "firstName": first_name,
        "lastName": last_name,
        "name": f"{first_name} {last_name}",
        "dateOfBirth": date_of_birth,
        "phone": phone,
        "email": username,
        "address1": address1,
        "address2": address2,
        "city": city,
        "state": state,
        "country": country,
        "postalCode": postal_code,
        "website": website,
        "random_word": random_word
    }
    return context

def fill_context_from_username(username):
    try:
        user = Client.get(email=username)
        context = {
            "locationId": user.leadmo_location_id,
            "calendarId": user.leadmo_calendar_id,
            "contactId": "",
            "firstName": user.first_name,
            "lastName": user.last_name,
            "name": f"{user.first_name} {user.last_name}",
            "dateOfBirth": user.date_of_birth,
            "phone": user.phone,
            "email": username,
            "address1": user.address1,
            "address2": user.address2,
            "city": user.city,
            "state": user.state,
            "country": user.country,
            "postalCode": user.postal_code,
            "website": user.website,
        }
    except Exception as e:
        logger.warning(e)
        context = {
            "locationId": "UNKNOWN",
            "email": username,
        }
    return context


def alert_on_actions(_session_state):
    """Alert on actions"""
    for i in range(3):
        time.sleep(4)
        logger.info(f"Checking for actions({i})...")

        try:
            _session_id = _session_state["session_id"]
            logger.info("Checking for actions: Found session_id")
        except TypeError:
            logger.warning("No session ID found")

        try:
            dynamodb = boto3.resource("dynamodb")
            table = dynamodb.Table(config.ACTION_TABLE_NAME)
            response = table.scan(
                FilterExpression=Attr('session_id').eq(_session_id)
            )
            logger.info(f"Checking for actions: {_session_id}")

            ret: dict = response.get("Items", {})
            logger.info(f"found ({len(ret)}) actions")
            for item in ret:
                event = item.get("event", "")
                response = item.get("response", "")
                logger.info(f"Event: {event}")
                logger.info(f"Response: {response}")
                if item["action_id"] in _session_state.get("events"):
                    continue
                _session_state.get("events").append(item["action_id"])
                gr.Info(f"Event: {event}\nResponse: {response}")
        except Exception as e:
            logger.error(f"Error checking for actions: {e}")
            pass

    return _session_state

